<?php
/*
Plugin Name: Post Revision System
Description: Systém pro revizi všech příspěvků pověřenými uživateli před jejich publikováním. <strong>Vyvinuto a určeno pro Pure Art.</strong> 
Version: 1.2
Author: Zaantar
Author URI: http://zaantar.eu
Plugin URI: http://zaantar.eu/index.php?page=post-revision-system
License: GPL2
*/

/*
    Copyright 2011 Zaantar (email: zaantar@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/



/*****************************************************************************\
		LOGGING AND NOTICES
\*****************************************************************************/


function prs_nag( $message ) {
	echo( '<div id="message" class="updated"><p>'.$message.'</p></div>' );
}

function prs_nagerr( $message ) {
	echo( '<div id="message" class="error"><p>'.$message.'</p></div>' );
}

function prs_log( $message, $category = 1 ) {
	if( !defined( 'WLS' ) or !wls_is_registered( 'post-revision-system' ) ) {
		$filename = dirname(__FILE__).'/log.txt';
		$file = fopen( $filename, 'ä́' );
		fwrite( $file, $message );
		fclose( $file );
	} else {
		wls_simple_log( 'post-revision-system', $message, $category );
	}
}



/*****************************************************************************\
		OPTIONS
\*****************************************************************************/


add_action( 'admin_menu','prs_admin_menu' );


function prs_admin_menu() {
	add_submenu_page( 'options-general.php', 'Nastavení Post Revision System', 'Post Revision System', 'manage_options', 
		'prs-options', 'prs_options_page' );
}


function prs_options_page() {
	if( isset($_REQUEST['action']) ) {
        $action = $_REQUEST['action'];
    } else {
        $action = 'default';
    }
    
    switch( $action ) {
    case 'wls-register':
    	$ok = wls_register( 'post-revision-system', 'Systémové záznamy pluginu Post Revision System.' );
    	if( $ok ) {
    		$info = 'Post Revision System byl úspěšně zaregistrován k WLS.';
    		prs_nag( $info );
    		prs_log( $info, 2 );
    	} else {
    		$info = 'Při pokusu o registraci Post Revision System k WLS došlo k chybě.';
    		prs_nagerr( $info );
    		prs_log( $info, WLS_ERROR );
    	}
    	prs_options_page_default();
    	break;
    case 'wls-unregister':
    	$ok = wls_unregister( 'post-revision-system' );
    	if( $ok ) {
    		$info = 'Post Revision System byl úspěšně odregistrován z WLS, systémové logy byly smazány.';
    		prs_nag( $info );
    		prs_log( $info, 2 );
    	} else {
    		$info = 'Při pokusu o odregistraci Post Revision System z WLS došlo k chybě.';
    		prs_nagerr( $info );
    		prs_log( $info, WLS_ERROR );
    	}
    	prs_options_page_default();
    	break;
	default:
		prs_options_page_default();
		break;
	}
}


function prs_options_page_default() {
	?>
	<div id="wrap">
		<h2>Nastavení Post Revision System</h2>
		<?php
			if( defined( 'WLS' ) ) {
				?>			
				<h3>Registrace do systémového logu WLS</h3>
		    	<p>Byl detekován nainstalovaný plugin Wordpress Logging Service.</p>
	    		<?php
	    			if( !wls_is_registered( 'post-revision-system' ) ) {
	    				?>
						<form method="post" action="options-general.php?page=prs-options">
							<input type="hidden" name="action" value="wls-register" />
							<p class="submit">
								<input type="submit" value="Registrovat Post Revision System do WLS" />    
							</p> 
						</form>
						<?php
					} else {
						?>
						<form method="post" action="options-general.php?page=prs-options">
							<input type="hidden" name="action" value="wls-unregister" />
							<p class="submit">
								<input type="submit" style="color:red;" value="Odregistrovat Post Revision System z WLS a nevratně smazat všechny logy" />    
							</p> 
						</form>
						<?php
					}
				?>
				<?php
        	}
        ?>
	</div>
	<?php
}


function prs_translate_revision_state( $state ) {
	switch( $state ) {
	case 'pending':
		return 'Čeká na přiřazení korektora.';
		break;
	case 'assigned':
		return 'Korektor přidělen, čeká na revizi.';
		break;
	case 'revised':
		return 'Připraveno ke zveřejnění.';
		break;
	default:
		return 'Nedefinovaný stav revize.';
		break;
	}
}


/*****************************************************************************\
		REVISION META BOX
\*****************************************************************************/

add_action( 'add_meta_boxes', 'prs_add_meta_box', 10, 2 );


function prs_add_meta_box( $arg, $post ) {
	if( $post->post_status == 'pending' ) {
		add_meta_box( 'post-revision-system', 'Revize článku', 'prs_meta_box', 'post', 'side', 'high' );
	}
}


function prs_meta_box( $post ) {
	if( $post->post_status != 'pending' ) {
		return;
	}
	$revision_state = get_post_meta( $post->ID, 'prs_revision_state', true );
	
	switch( $revision_state ) {
	case 'pending':
		if( current_user_can( 'assign_revisor' ) ) {
			$revisors = prs_get_revisors( $post->post_author );
			?>
			<p>Tento článek můžete přidělit ke korektuře:</p>
			<select name="prs_assign_revisor">
				<option value="0" selected="selected">- nepřidělovat -</option>
				<?php
					foreach( $revisors as $revisor ) {
						echo '<option value="'.$revisor->ID.'">'.$revisor->user_login.'</option>';
					}
				?>
			</select>
			<?php 
			submit_button( __( 'Save' ), 'button', 'save' );
			
		} else if( current_user_can( 'revisor' ) ) {
			if( $post->post_author == get_current_user_id() ) {
				?>
				<p>Toto je váš článek, který čeká na přidělení korektora.</p>
				<p><em>Přestože jste také korektor, není dovoleno převzít ke korektuře vlastní článek.</em></p>
				<?php
			} else {
				?>
				<p>Tento článek dosud nebyl přidělen žádnému korektorovi.</p>
				<input type="checkbox" name="prs_assign_self" /><label for="prs_assign_self"><strong>Převzít korekturu článku</strong></label>
				<?php
				submit_button( __( 'Save' ), 'button', 'save' );
			}
		} else {
			?>
			<p>Tento článek čeká na přidělení korektora.</p>
			<?php
		}	
				
		remove_meta_box( 'submitdiv', 'post', 'side' );
		
		break;
		
	case 'assigned':
	
		$revisor_id = get_post_meta( $post->ID, 'revisor_id', true );
		if( $revisor_id == get_current_user_id() ) {
			?>
			<p>Tento článek čeká na vaši korekturu. Proveďte, prosím, nezbytné úpravy a předejte jej ke zveřejnění.</p>
			<input type="checkbox" name="prs_mark_revised" /><label for="prs_mark_revised"><strong>Korektura je hotova</strong></label>
			<?php
			submit_button( __( 'Save' ), 'button', 'save' );
		} else if( current_user_can( 'assign_revisor' ) ) {
			$revisor = get_userdata( $revisor_id );
			?>
			<p>Tento článek je přidělen korektorovi <strong><?php echo $revisor->user_login; ?></strong>.</p>
			<label for="prs_set_revision_state">Snížit stav revize:</label>
			<select name="prs_set_revision_state">
				<option value="not" selected="selected">- nesnižovat -</option>
				<option value="pending">Zrušit přiřazení korektorovi</option>
			</select>
			<?php
			submit_button( __( 'Save' ), 'button', 'save' );
		} else {
			?>
			<p>Tento článek je již přidělen korektorovi.</p>
			<?php
		}
		
		remove_meta_box( 'submitdiv', 'post', 'side' );
		
		break;
		
	case 'revised':
		
		if( current_user_can( 'assign_revisor' ) ) {
			$revisor_id = get_post_meta( $post->ID, 'revisor_id', true );
			$revisor = get_userdata( $revisor_id );
			?>
			<p>Tento článek byl zrevidován korektorem <strong><?php echo $revisor->user_login; ?></strong> a je připraven ke zveřenění šéfredaktorem.</p>
			<label for="prs_set_revision_state">Snížit stav revize:</label>
			<select name="prs_set_revision_state">
				<option value="not" selected="selected">- nesnižovat -</option>
				<option value="assigned">Zrušit označení za zrevidované</option>
				<option value="pending">Zrušit přiřazení korektorovi</option>
			</select>
			<?php
			submit_button( __( 'Save' ), 'button', 'save' );
		} else {
			?>
			<p>Tento článek byl zrevidován a je připraven ke zveřenění šéfredaktorem.</p>
			<?php
		}
		if( !current_user_can( 'publish_posts' ) /* or ( $revisor_id == get_current_user_id() ) */ ) {
			remove_meta_box( 'submitdiv', 'post', 'side' );
		}
		
		break;
	default: 
		return;
	}
	
	wp_nonce_field( plugin_basename( __FILE__ ), 'prs_save_nonce' );
	
}


add_action( 'save_post', 'prs_save_postdata' );


function prs_save_postdata( $post_id, $nononce = false ) {
	if( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) { 
		prs_log( 'exiting prs_save_postdata( '.$post_id.' ) on DOING_AUTOSAVE.', 1 );
		return;
	}
	if( !$nononce ) {
		if( !wp_verify_nonce( $_POST['prs_save_nonce'], plugin_basename( __FILE__ ) ) ) {
			prs_log( 'exiting prs_save_postdata( '.$post_id.' ) on wp_verify_nonce.', 1 );
			return;
		}
	}
	if( $_POST['post_type'] != 'post' ) {
		prs_log( 'exiting prs_save_postdata( '.$post_id.' ) on $_POST[\'post_type\'] != \'post\'.', 1 );
		return;
	}
	if( !current_user_can( 'edit_post' ) ) {
		prs_log( 'exiting prs_save_postdata( '.$post_id.' ) on !current_user_can( \'edit_post\' ).', 1 );
		return;
	}
	
	$post = get_post( $post_id );
	if( $post->post_status == 'inherit' ) {
		prs_log( 'post id=='.$post_id.' is inherited from '.$post->post_parent.', redirecting.' );
		return prs_save_postdata( $post->post_parent, true );
	}
		
	if( $post->post_status != 'pending' ) {
		prs_log( 'exiting prs_save_postdata( '.$post_id.' ) on $post->post_status != \'pending\'.', 1 );
		return;
	}
	
	$revision_state = get_post_meta( $post->ID, 'prs_revision_state', true );
		
	if( ( $revision_state == 'pending' ) && isset( $_POST['prs_assign_self'] ) && $_POST['prs_assign_self'] && current_user_can( 'revisor' ) ) {
		$authordata = get_userdata( $post->post_author );
		add_post_meta( $post_id, 'revisor_id', get_current_user_id(), true ) or update_user_meta( $post_id, 'revisor_id', get_current_user_id() );
		add_post_meta( $post_id, 'prs_revision_state', 'assigned', true ) or update_post_meta( $post_id, 'prs_revision_state', 'assigned' );
		prs_log( 'Revisor has assumed post '.$post_id.' ("'.$post->post_title.'" by '.$authordata->user_login.') for revision.', 2 );
	}
	
	if( ( $revision_state == 'assigned' ) && isset( $_POST['prs_mark_revised'] ) && $_POST['prs_mark_revised'] && current_user_can( 'revisor' ) ) {
		$post = get_post( $post_id );
		$authordata = get_userdata( $post->post_author );
		add_post_meta( $post_id, 'prs_revision_state', 'revised', true ) or update_post_meta( $post_id, 'prs_revision_state', 'revised' );
		prs_log( 'Revisor has marked post '.$post_id.' ("'.$post->post_title.'" by '.$authordata->user_login.') as revised.', 2 );
	}
		
	if( ( $revision_state == 'pending' ) && isset( $_POST['prs_assign_revisor'] ) && current_user_can( 'assign_revisor' ) ) {
		$revisor_id = $_POST['prs_assign_revisor'];
		$post = get_post( $post_id );
		$authordata = get_userdata( $post->post_author );
		add_post_meta( $post_id, 'revisor_id', $revisor_id, true ) or update_user_meta( $post_id, 'revisor_id', $revisor_id );
		add_post_meta( $post_id, 'prs_revision_state', 'assigned', true ) or update_post_meta( $post_id, 'prs_revision_state', 'assigned' );
		$revisordata = get_userdata( $revisor_id );
		prs_log( 'User has assigned post '.$post_id.' ("'.$post->post_title.'" by '.$authordata->user_login.') to revisor '.$revisordata->user_login.'.', 2 );
	}
	
	if( isset( $_POST['prs_set_revision_state'] ) 
		&& ( $_POST['prs_set_revision_state'] != 'not' )
		&& current_user_can( 'assign_revisor' ) ) {
		
		add_post_meta( $post_id, 'prs_revision_state', $_POST['prs_set_revision_state'], true ) or update_post_meta( $post_id, 'prs_revision_state', $_POST['prs_set_revision_state'] );
		
		$post = get_post( $post_id );
		$authordata = get_userdata( $post->post_author );
		prs_log( 'User has set revision state "'.$_POST['prs_set_revision_state'].'" for post '.$post_id.' ("'.$post->post_title.'" by '.$authordata->user_login.').', 2 );
	}
}

$prs_revisor_cache;
$prs_revisor_cache_set = false;

function prs_get_revisors( $exclude_user = 0 ) {
	// toto je oskliva funkce, kterou nechceme a nemusime volat vicekrat behem jednoho spusteni skriptu
	if( $prs_revisor_cache_set ) {
		return $prs_revisor_cache;
	}
	
	// v usermeta wp_?_capabilities se ukladaji pouze nazvy roli a externe pridane capabilities
	// musime tedy nejdrive vytvorit seznam vsech roli, ktere obsahuji capability revisor\
	// a nasledne provest vyhledavani nad vsemi temito rolemi (a sloucit vysledky)
	$wp_roles = new WP_Roles();
	$roles = array();
	//print_r( $wp_roles->roles );
	foreach( $wp_roles->roles as $rolename => $role ) {
		if( array_key_exists( 'revisor', $role['capabilities'] ) && ( $role['capabilities']['revisor'] == 1 ) ) {
			$roles[] = $rolename;
		}
	}
	
	$revisors = array();
	$roles[] = 'revisor'; // pokud ma nekdo capability pridanou mimo obvykle role
	foreach( $roles as $role ) {
		$revisor_search = new WP_User_Query( array( 'role' => $role, 'fields' => array( 'ID', 'user_login' ) ) );
		$revisor_results = $revisor_search->get_results();
		foreach( $revisor_results as $revisor_result ) {
			//echo '<p>found revisor: '.print_r( $revisor_result, true ).'.</p>';
			if( !in_array( $revisor_result, $revisors ) ) {
				if( !$exclude_user or ( $revisor_result->ID != $exclude_user ) ) {
					$revisors[] = $revisor_result;
				}
			}
		}
	}
	
	// jeste revisory setridime
	usort( $revisors, 'prs_sort_revisors' );
	
	$prs_revisor_cache = $revisors;
	$prs_revisor_cache_set = true;
	return $revisors;
}


function prs_sort_revisors( $r1, $r2 ) {
	return strcmp( $r1->user_login, $r2->user_login );
}


/*****************************************************************************\
		FILTER OTHER'S POSTS FOR REVISOR
\*****************************************************************************/

add_action( 'save_post', 'prs_ensure_revision_state' );

function prs_ensure_revision_state( $post_id ) {
	prs_log( 'calling ensure_revision_state for post '.$post_id.'.', 1 );
	$post = get_post( $post_id );
	if( $post->post_status == 'inherit' ) {
		prs_log( 'ensure_revision_state: inherited post '.$post_id.' from '.$post->post_parent.', redirecting.', 1 );
		return prs_ensure_revision_state( $post->post_parent );
	} else {
		prs_log( 'ensure_revision_state: post status is '.$post->post_status.', continuing.' );
	}
	$revision_state = get_post_meta( $post_id, 'prs_revision_state', true );
	if( ( $revision_state == '' ) or ( $revision_state == 'none' ) ) {
		prs_log( 'ensure_revision_state: no revision state for post '.$post_id.', set to \'pending\'.', 1 );
		add_post_meta( $post_id, 'prs_revision_state', 'pending', true ) or update_post_meta( $post_id, 'prs_revision_state', 'pending' );
	}
}

/* from Manage Your Posts Only plugin: http://code.mincus.com/41/manage-your-posts-only-in-wordpress/ */

add_filter( 'parse_query', 'prs_parse_query_useronly' );


function prs_parse_query_useronly( $wp_query ) {
	global $pagenow;
    if ( $pagenow == 'edit.php' ) {

    	if( isset( $_GET['prs_filter_revised_state'] ) && ( $_GET['prs_filter_revised_state'] != 'all' ) ) {

    		$filter = $_GET['prs_filter_revised_state'];
    		switch( $filter ) {
    		case 'pending':
    			$meta_query = array( 
    				array(
						'key' => 'prs_revision_state',
						'compare' => 'LIKE',
						'value' => 'pending'
					),
				);
    			
    			break;
    		default:
    			$meta_query = array(
    				'relation' => 'AND',
    				array(
    				 	'key' => 'prs_revision_state',
        				'compare' => 'LIKE',
        				'value' => $filter
        			),
		        );
		        if( current_user_can( 'revisor_only' ) && !current_user_can( 'manage_options' ) ) {
		        	$meta_query[] = array(
						'key' => 'revisor_id',
						'compare' => '=',
						'value' => get_current_user_id(),
						'type' => 'NUMERIC'
		        	);
				}
    			break;
    		}
    		
    		$wp_query->set( 'meta_query', $meta_query );
    		$wp_query->set( 'post_status', $pending );
    		
        } else if( current_user_can( 'revisor_only' ) && !current_user_can( 'manage_options' ) ) {

        	$wp_query->set( 'post_status', 'pending' );
        	$meta_query = $wp_query->get( 'meta_query' );
        	if( empty( $meta_query ) ) {
        		$meta_query = array();
        	}
        	if( isset( $meta_query['relation'] ) ) {
        		prs_log( 'Overriding $meta_query[\'relation\'] value "'.$meta_query['relation'].'" to "OR".', 3 );
        	}
			$meta_query['relation'] = 'OR';
        	$meta_query[] = array(
        		'key' => 'revisor_id',
        		'compare' => '=',
        		'value' => get_current_user_id(),
        		'type' => 'NUMERIC'
        	);
        	$meta_query[] = array(
        		'key' => 'prs_revision_state',
        		'compare' => 'LIKE',
        		'value' => 'pending',
        	);
        	$wp_query->set( 'meta_query', $meta_query );

        } 
        
        if( !current_user_can( 'edit_others_posts' ) ) {

            global $current_user;
            $wp_query->set( 'author', $current_user->id );

		}
    }
}


/* from Bainternet Posts Creation Limits http://en.bainternet.info/category/plugins */

add_action( 'admin_head', 'prs_disable_revisor_post_creation' );


function prs_disable_revisor_post_creation() {
	global $pagenow;
	if( ( $pagenow == 'post-new.php' ) && current_user_can( 'revisor_only' ) && !current_user_can( 'manage_options' ) ) {
		prs_nagerr( 'S tímto uživatelským účtem v roli korektora nemáte povoleno vytvářet vlastní příspěvky. <a href="index.php">Zpět do administrace.</a>' );
		exit;
	}
	
}


add_action( 'admin_head', 'prs_remove_revisor_newpost_submenu' );


function prs_remove_revisor_newpost_submenu() {
	global $submenu;
	if( current_user_can( 'revisor_only' ) && !current_user_can( 'manage_options' ) ) {
		unset( $submenu['edit.php'][10] );
	}
}


/*****************************************************************************\
		REVISION STATE FILTERS IN POST MANAGEMENT PAGE
\*****************************************************************************/


add_action( 'restrict_manage_posts', 'prs_restrict_manage_posts' );


function prs_restrict_manage_posts() {
	global $post_type;
	if( $post_type != 'post' ) {
		return;
	}
	
	?>
	<label for="prs_filter_revised_state"><strong>Stav revize článku:</strong></label>
	<select name="prs_filter_revised_state">
		<option value="all" selected="selected">Libovolný</option>
		<option value="pending">Čekající na přiřazení</option>
		<option value="assigned">Přiřazen korektorovi</option>
		<option value="revised">Připraven ke zveřejnění</option>
	</select>
	<?php
}


/*****************************************************************************\
		ADD COLUMN IN POST MANAGEMENT
\*****************************************************************************/


add_filter( 'manage_posts_columns' , 'prs_posts_custom_columns' );


function prs_posts_custom_columns( $columns ) {
	if( current_user_can( 'edit_others_posts' ) 
		&& ( !current_user_can( 'revisor_only' ) or current_user_can( 'manage_options' ) ) ) 
	{
	    $new_columns = array_merge( $columns, array( 'revision_info' => 'Stav revize<br />Přidělený korektor' ) );
		return $new_columns;
	} else {
		return $columns;
	}
}


add_action( 'manage_posts_custom_column' , 'prs_post_custom_column' );

function prs_post_custom_column( $column ) {
	global $post;
	if( $column == 'revision_info' ) {
		if( $post->post_status == 'pending' ) {
			$revision_state = get_post_meta( $post->ID, 'prs_revision_state', true );
			$revisor_id = get_post_meta( $post->ID, 'revisor_id', true );
			if( ( $revisor_id != '' ) && ( $revisor_id != 0 ) && ( $revision_state != 'pending' ) ) {
				$revisordata = get_userdata( $revisor_id );
				$revisor_name = $revisordata->user_login;
			} else {
				$revisor_name = '<small>korektor nepřidělen</small>';
			}
			echo prs_translate_revision_state( $revision_state ).'<br />'.$revisor_name;
		}
	}
}

/*****************************************************************************\
		ADD COLUMN IN USER LIST
\*****************************************************************************/


add_filter( 'manage_users_columns', 'prs_add_revision_count_column');


function prs_add_revision_count_column( $columns ){
    $columns['revised_posts'] = 'Počet opravovaných článků';
    return $columns;
}


add_action( 'manage_users_custom_column',  'prs_add_revision_count_column_value', 10, 3);


function prs_add_revision_count_column_value( $value, $column_name, $user_id ){
	if( !current_user_can( 'assign_revisor' ) ) {
		return;
	}
	if( 'revised_posts' == $column_name && user_can( $user_id, 'revisor' ) ) {
		$wp_query = new WP_Query( array( 
			'meta_query' => array(
				array(
					'key' => 'revisor_id',
					'value' => $user_id,
					'type' => 'NUMERIC'
				),
				'relation' => 'AND',
				array(
					'key' => 'prs_revision_state',
					'value' => 'revised'
				)
			)
		) );
		$wp_query->get_posts();
		return $wp_query->post_count;	
	}
	
}



?>
